const Filter = require('broccoli-persistent-filter');
const crypto = require('crypto');
const fs = require('fs');
const path = require('path');

const wabt = require('./vendor/libwabt.js');

const checksum = crypto.createHash('md5')
  .update(fs.readFileSync(
    path.join(__dirname, 'vendor/libwabt.js')))
  .digest('hex');

class Wat2Wasm extends WabtFilter {
  constructor(input, options = {}) {
    super(input, options);
    this.readDebugNames = options.readDebugNames !== false;
    this.generateNames = options.generateNames !== false;
    this.foldExprs = options.foldExprs !== false;
    this.inlineExport = options.inlineExport !== false;
    this.extensions = ['.wat', '.wast'];
    this.targetExtension = '.wasm';
    this.inputEncoding = 'utf8';
    this.outputEncoding = null;
  }

  /**
   * Parse wat and output wasm.
   * @param {string} wat
   * @param {string} relativePath
   * @returns {Uint8Array} the wasm module bytes
   */
  processString(wat, relativePath) {
    return wabt.ready.then(() => {
      const mod = wabt.parseWat(relativePath, wat);
      try {
        mod.resolveNames();
        mod.validate();
        return mod.toBinary({log: false}).buffer;
      } finally {
        mod.dispose();
      }
    });
  }
}

class Wasm2Wat extends WabtFilter {
  constructor(input, options) {
    super(input, options);
    this.extensions = ['.wasm'];
    this.targetExtension = '.wat';
    this.inputEncoding = null;
    this.outputEncoding = 'utf8';
  }

  /**
   * Read wasm and output wat.
   * @param {Buffer} wasm
   * @returns {string} wat
   */
  processString(wasm) {
    return wabt.ready.then(() => {
      const mod = wabt.readWasm(wasm, { readDebugNames: this.readDebugNames });
      try {
        if (this.generateNames) {
          mod.generateNames();
          mod.applyNames();
        }
        return mod.toText({ foldExprs: this.foldExprs, inlineExport: this.inlineExport });
      } finally {
        mod.dispose();
      }
    }
  }
}

class WabtFilter extends Filter {
  constructor(input, options) {
    let annotation;
    if (options !== undefined) {
      annotation = options.annotation;
    }
    super(input, {
      annotation,
      persist: true,
      async: true,
    });
  }

  cacheKey() {
    return checksum;
  }
}

module.exports.Wat2Wasm = Wat2Wasm;
module.exports.Wasm2Wat = Wasm2Wat;
